﻿using System;
using System.Collections.Generic;
using System.Timers;
using Ozeki.Media;
using Ozeki.VoIP;

namespace CallQueueAdvanced
{
    class CallQueueCallHandler
    {
        ICall call;
        Timer timer;
        Timer timerAgent;
        ICallManager callManager;
        IExtensionContainer extensionContainer;
        List<string> members;

        MediaConnector mediaConnector;
        PhoneCallAudioSender phoneCallAudioSender;
        MP3StreamPlayback mp3Player;
        TextToSpeech textToSpeech;

        CallQueueHandlerContainer callQueueHandlerContainer;
        double waitingTime;

        public CallQueueCallHandler(ICall call, ICallManager callManager, IExtensionContainer extensionContainer, List<string> members, CallQueueHandlerContainer callQueueHandlerContainer)
        {
            timer = new Timer(1000);
            timer.AutoReset = true;
            timer.Elapsed += timer_Elapsed;

            timerAgent = new Timer(15000);
            timerAgent.AutoReset = true;
            timerAgent.Elapsed += timerAgent_Elapsed;

            this.members = members;
            this.callManager = callManager;
            this.extensionContainer = extensionContainer;
            this.callQueueHandlerContainer = callQueueHandlerContainer;

            this.call = call;
            this.call.CallStateChanged += call_CallStateChanged;

            mediaConnector = new MediaConnector();
            phoneCallAudioSender = new PhoneCallAudioSender();
            mp3Player = new MP3StreamPlayback(@"test.mp3", true, true);
            phoneCallAudioSender.AttachToCall(call);

            textToSpeech = new TextToSpeech();
            textToSpeech.Stopped += TextToSpeechCompleted;

            mediaConnector.Connect(mp3Player, phoneCallAudioSender);
            mediaConnector.Connect(textToSpeech, phoneCallAudioSender);

            timer.Start();
            timerAgent.Start();
        }

        private bool blindTransferEnabled;
        public void Start()
        {
            if (call.CallState.IsCallEnded())
            {
                OnCompleted();
                return;
            }

            blindTransferEnabled = true;
        }

        private void OnCompleted()
        {
            mediaConnector.Dispose();
            mp3Player.Dispose();
            textToSpeech.Dispose();

            var handler = Completed;
            if (handler != null)
                handler(this, EventArgs.Empty);
        }
        
        public event EventHandler Completed;

        void timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (!call.CallState.IsInCall())
                return;

            var memb = new List<string>(members);
            
            foreach (var activeSession in callManager.ActiveSessions)
            {
                var callee = activeSession.CalleeInfo.Owner.ExtensionID;
                var caller = activeSession.CallerInfo.Owner.ExtensionID;

                memb.Remove(callee);
                memb.Remove(caller);

                double sum = 0;
                var callCounter = 0;
                
                foreach (var m in memb)
                {
                    List<ISession> historyResult = CallHistory.GetCallHistory(m);

                    foreach (var session in historyResult)
                    {
                        ++callCounter;
                        sum += session.TalkDuration.TotalSeconds;
                    }
                }

                if (sum != 0)
                {
                    waitingTime = sum / callCounter;
                }
            }

            if(!blindTransferEnabled)
                return;

            foreach (var member in memb)
            {
                if (extensionContainer.GetExtension(member) != null)
                {
                    call.BlindTransfer(member);
                }
            }
        }

        void timerAgent_Elapsed(object sender, ElapsedEventArgs e)
        {
            mp3Player.Pause();
            textToSpeech.Clear();

            if (waitingTime == 0)
            {
                textToSpeech.AddAndStartText(string.Format("{0} more customers are waiting in line before you. Please be patient.",
                                                           callQueueHandlerContainer.GetIndex(this)));
            }
            else
            {
                textToSpeech.AddAndStartText(string.Format("{0} more customers are waiting in line before you. " +
                                                           "Your estimated wait time is approximately {1} seconds. Please be patient.",
                                                           callQueueHandlerContainer.GetIndex(this),
                                                           (int)waitingTime));
            }
        }

        void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            if (e.State == CallState.Answered)
            {
                mp3Player.Start();
            }

            if (e.State.IsRinging())
            {
                call.Answer();
            }
            
            if (e.State.IsCallEnded())
            {
                OnCompleted();
            }
        }

        void TextToSpeechCompleted(object sender, EventArgs e)
        {
            mp3Player.Start();
            textToSpeech.Clear();
        }
    }
}
